#!/usr/bin/env python

"""
Operator support.

Copyright (C) 2010 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

def binary_op(a, b, left_name, right_name):

    """
    A single parameterised function providing the binary operator mechanism for
    arguments 'a' and 'b' using methods of the given 'left_name' and
    'right_name'.
    """

    # First, try and get a method for the left argument, and then call it with
    # the right argument.

    try:
        fn = getattr(a, left_name)
    except AttributeError:
        pass
    else:
        result = fn(b)
        if result is not NotImplemented:
            return result

    # Otherwise, try and get a method for the right argument, and then call it
    # with the left argument.

    try:
        fn = getattr(b, right_name)
    except AttributeError:
        pass
    else:
        result = fn(a)
        if result is not NotImplemented:
            return result

    # Where no methods were available, or if neither method could support the
    # operation, raise an exception.

    raise TypeError

def add(a, b):
    return binary_op(a, b, "__add__", "__radd__")

def and_(a, b):
    return binary_op(a, b, "__and__", "__rand__")

def div(a, b):
    return binary_op(a, b, "__div__", "__rdiv__")

def floordiv(a, b):
    return binary_op(a, b, "__floordiv__", "__rfloordiv__")

def lshift(a, b):
    return binary_op(a, b, "__lshift__", "__rlshift__")

def mod(a, b):
    return binary_op(a, b, "__mod__", "__rmod__")

def mul(a, b):
    return binary_op(a, b, "__mul__", "__rmul__")

def or(a, b):
    return binary_op(a, b, "__or__", "__ror__")

def pow(a, b):
    return binary_op(a, b, "__pow__", "__rpow__")

def rshift(a, b):
    return binary_op(a, b, "__rshift__", "__rrshift__")

def sub(a, b):
    return binary_op(a, b, "__sub__", "__rsub__")

def xor(a, b):
    return binary_op(a, b, "__xor__", "__rxor__")

# vim: tabstop=4 expandtab shiftwidth=4
